class QRadarModel:
    @classmethod
    def from_json(cls, json):
        if isinstance(json, dict):
            obj = cls()
            for att in vars(obj):
                if att is not None:
                    setattr(obj, att, json.get(att, None))
            return obj
        if isinstance(json, list):
            listofitems = []
            for item in json:
                obj = cls()
                for att in vars(obj):
                    if att is not None:
                        setattr(obj, att, item.get(att, None))
                listofitems.append(obj)
            return listofitems
        else:
            raise ValueError

    def __repr__(self):
        return str(self.__class__) + ": " + str(self.__dict__)



from .qradarmodel import QRadarModel

class Domain(QRadarModel):

    def __init__(self, *, id=None, name=None, description=None, tenant_id=None, deleted=None, event_collector_ids=None, log_source_ids=None,
                 log_source_group_ids=None, custom_properties=None, flow_source_ids=None, flow_collector_ids=None, asset_scanner_ids=None, qvm_scanner_ids=None, flow_vlans_ids=None):

        self.id = id
        self.name = name
        self.description = description
        self.tenant_id = tenant_id
        self.deleted = deleted
        self.event_collector_ids = event_collector_ids
        self.log_source_ids = log_source_ids
        self.log_source_group_ids = log_source_group_ids
        self.custom_properties = custom_properties
        self.flow_source_ids = flow_source_ids
        self.flow_collector_ids = flow_collector_ids
        self.asset_scanner_ids = asset_scanner_ids
        self.qvm_scanner_ids = qvm_scanner_ids
        self.flow_vlans_ids = flow_vlans_ids




import requests


class QRadarAPIClient:
    """"
    The parent class for a QRadar API endpoints providing a constructor and several classmethods.
    """

    def __init__(self, baseurl, header, verify=True):
        self._baseurl = baseurl
        self._header = header
        self._verify = verify

    def get_header(self):
        """
        Return the header to use in API calls.
        :return: The header defined while constructing the class.
        """
        return self._header

    def _call(self, method, url, **kwargs):
        """
        Wrapper around the requests request method.
        :param method:  The HTTP method to use.
        :param url:     The url to query.
        :param kwargs:  The arguments to pass to requests.
        :return:        The status code and the response in JSON.
        """
        header = self._header.copy()
        header.update(kwargs.pop('headers', {}))
        header.update(kwargs.pop('mime_type', {}))

        response = requests.request(
            method, url, headers=header, verify=self._verify, **kwargs)
        response.raise_for_status()
        return response.json()

        


def header_vars(*valid_header_fields):
    """
    Decorator to assign variables to the header field.
    :param valid_header_fields: The variables to move from **kwargs to header.
    :return: A decorator that checks for the passed variables and moves them to the header var.
    """
    def decorator(func):
        def wrapped(self, *args, **all_args):
            header = all_args.pop('headers', {})
            valid_header = {key: all_args[key]
                            for key in all_args if key in valid_header_fields}
            header.update(valid_header)
            return func(self, *args, headers=header, **all_args)
        return wrapped
    return decorator


def request_vars(*valid_param_fields):
    """
    Decorator to assign variables to the params field.
    :param valid_param_fields:  The variables to move from **kwargs to params.
    :return: A decorator that checks for the passed variables and moves them to the params var.
    """
    def decorator(func):
        def wrapped(self, *args, **all_args):
            params = all_args.pop('params', {})
            valid_params = {key: all_args[key]
                            for key in all_args if key in valid_param_fields}
            params.update(valid_params)
            return func(self, *args, params=params, **all_args)
        return wrapped
    return decorator



from .qradarmodel import QRadarModel


class Logsource(QRadarModel):
    def __init__(self, *, id=None, name=None, description=None, type_id=None, protocol_type_id=None, protocol_parameters=None, enabled=None, group_ids=None, average_eps=None):
        self.id = id
        self.name = name
        self.description = description
        self.type_id = type_id
        self.protocol_type_id = protocol_type_id
        self.protocol_parameters = protocol_parameters
        self.enabled = enabled
        self.group_ids = group_ids
        self.average_eps = average_eps



from .qradarmodel import QRadarModel

class LogsourceGroup(QRadarModel):

    def __init__(self, *, id=None, name=None, description=None, parent_id=None, owner=None, modification_date=None, child_groups=None):
        self.id = id
        self.name = name
        self.description = description
        self.parent_id = parent_id
        self.owner = owner
        self.modification_date = modification_date
        self.child_groups = child_groups



from urllib.parse import urljoin

from qradar.api.client import QRadarAPIClient
from qradar.api.client import header_vars as headers
from qradar.api.client import request_vars as params
from qradar.models import Logsource, LogsourceGroup, Domain


class Config(QRadarAPIClient):
    """
    The QRadar API endpoint group /config and its endpoints.
    """
    __baseurl = 'config/'

    def __init__(self, url, header, verify):
        super().__init__(urljoin(url, self.__baseurl),
                         header,
                         verify)

    @headers('Range')
    @params('filter', 'fields')
    def get_log_source_groups(self, *, filter=None, fields=None, Range=None, **kwargs):
        """
        GET - /config/event_sources/log_source_management/log_source_groups
        Retrieve a list of all log source groups
        """
        function_endpoint = urljoin(
            self._baseurl, 'event_sources/log_source_management/log_source_groups')
        return LogsourceGroup.from_json(self._call('GET', function_endpoint, **kwargs))

    @headers('Range')
    @params('filter', 'fields', 'sort')
    def get_log_sources(self, *, filter=None, fields=None, Range=None, sort=None, **kwargs):
        """
        GET - /config/event_sources/log_source_management/log_sources
        Retrieves a list of log sources
        """
        function_endpoint = urljoin(
            self._baseurl, 'event_sources/log_source_management/log_sources')
        return Logsource.from_json(self._call('GET', function_endpoint, **kwargs))

    @headers('Range')
    @params('filter', 'fields', 'sort')
    def get_domains(self, *, filter=None, fields=None, Range=None, **kwargs):
        """
        GET - /config/domain_management/domains
        Gets the list of domains. You must have the System Administrator or Security Administrator permissions to call this endpoint if you are trying to retrieve the details of all domains. 
        You can retrieve details of domains that are assigned to your Security Profile without having the System Administrator or Security Administrator permissions. 
        If you do not have the System Administrator or Security Administrator permissions, then for each domain assigned to your security profile you can only view the values for the id and name fields. All other values return null.
        """
        function_endpoint = urljoin(
            self._baseurl, 'domain_management/domains')
        return Domain.from_json(self._call('GET', function_endpoint, **kwargs))



from .qradarmodel import QRadarModel


class LoginAttempt(QRadarModel):
    def __init__(self, *, attempt_time=None, user_id=None, remote_ip=None, attempt_result=None, attempt_method=None):
        self.attempt_time = attempt_time
        self.user_id = user_id
        self.remote_ip = remote_ip
        self.attempt_result = attempt_result
        self.attempt_method = attempt_method



from inspect import isclass
from pkgutil import iter_modules
from pathlib import Path
from importlib import import_module

package_dir = Path(__file__).resolve().parent
for (_, module_name, _) in iter_modules([package_dir]):
    module = import_module(f"{__name__}.{module_name}")
    for attribute_name in dir(module):
        attribute = getattr(module, attribute_name)
        if isclass(attribute):
            globals()[attribute_name] = attribute



from urllib.parse import urljoin

from qradar.api.client import QRadarAPIClient
from qradar.api.client import header_vars as headers
from qradar.api.client import request_vars as params
from qradar.models import SavedSearchGroup


class Ariel(QRadarAPIClient):
    """
    The QRadar API endpoint group /ariel and its endpoints.
    """
    __baseurl = 'ariel/'

    def __init__(self, url, header, verify):
        super().__init__(urljoin(url, self.__baseurl),
                         header,
                         verify)

    @headers('Range')
    @params('filter', 'fields')
    def get_event_saved_search_groups(self, *, filter=None, fields=None, Range=None, **kwargs):
        """
        GET - /ariel/event_saved_search_groups
        Retrieves a list the event Ariel saved search groups
        """
        function_endpoint = urljoin(
            self._baseurl, 'event_saved_search_groups')
        return self._call('GET', function_endpoint, **kwargs)

    @headers('Range')
    @params('filter', 'fields')
    def get_flow_saved_search_groups(self, *, filter=None, fields=None, Range=None, **kwargs):
        """
        GET - /ariel/flow_saved_search_groups
        Retrieves a list the event Ariel saved search groups
        """
        function_endpoint = urljoin(
            self._baseurl, 'flow_saved_search_groups')
        return self._call('GET', function_endpoint, **kwargs)



from urllib.parse import urljoin

from qradar.api.client import QRadarAPIClient
from qradar.api.client import header_vars as headers
from qradar.api.client import request_vars as params
from qradar.models import LoginAttempt


class Access(QRadarAPIClient):
    """
    The QRadar API endpoint group /access and its endpoints.
    """
    __baseurl = 'access/'

    def __init__(self, url, header, verify):
        super().__init__(urljoin(url, self.__baseurl),
                         header,
                         verify)

    @headers('Range')
    @params('filter', 'fields')
    def get_login_attempts(self, *, filter=None, fields=None, Range=None, **kwargs):
        """
        GET - /access/login_attempts
        Gets the list of login attempts
        """
        function_endpoint = urljoin(
            self._baseurl, 'login_attempts')
        return LoginAttempt.from_json(self._call('GET', function_endpoint, **kwargs))



from .qradarmodel import QRadarModel


class SavedSearchGroup(QRadarModel):
    def __init__(self, *, id=None, parent_id=None, type=None, level=None, name=None, description=None,  owner=None, modified_time=None, child_group_ids=None):
        self.id = id
        self.parent_id = parent_id
        self.type = type
        self.level = level
        self.name = name
        self.description = description
        self.owner = owner
        self.modified_time = modified_time
        self.child_group_ids = child_group_ids
